// $Id: JareEnv.java,v 1.5 2001/08/12 18:23:12 ioerger Exp $

package Jare;
import java.util.Hashtable;
import java.util.Vector;

// keys are stored in upper case to make matching case-insensitive
//   but values are stored with their original case

public class JareEnv
{
  public Hashtable hash;

  public JareEnv()
  {  
    hash = new Hashtable();
  }

  public JareEnv(JareEnv e)
  {
    hash = (Hashtable)e.hash.clone();
  }

  // recursive
  // transitive for chained vars
  // non-destructive - creates copy

  public Object subst(Object a)
  {
    if (J.isString(a))
    {
      Object s=lookup((String)a);
      if (s!=null) return subst(s);
      else return a;
    }
    else
    {
      Vector v=(Vector)a;
      Vector w=new Vector();
      for (int i=0 ; i<v.size() ; i++)
        w.addElement(subst(v.elementAt(i)));
      return w;
    }
  }

  public void bind(String a,Object b)
  {
    hash.put(a.toUpperCase(),b);
  }

  // returns null if not found
  // NOT transitive
  // what if numeric?
 
  public Object lookup(String a)
  {
    return hash.get(a.toUpperCase()); 
  }

  // non-destructive - returns new binding environment or null

  JareEnv unify(Object a,Object b)
  {
    JareEnv e=new JareEnv(this);
    boolean result=e.unify1(a,b);
    if (result==true) return e;
    else return null;
  }

  // destructive - changes bindings for this object; return T or F

  boolean unify1(Object a,Object b)
  {
    // if either is a var, make sure it is first
    if (J.isString(b) && J.isVar((String)b))
    {
      Object temp;
      temp = a;
      a = b;
      b = temp; 
    }
    if (J.isString(a))
    {
      String p=(String)a;
      if (!J.isVar(p)) // then b can't be var, because of swap above
      { 
        // added to handle numerics: compare as floats (8/7/00)
        char c=p.charAt(0);
        if ((c=='+')||(c=='-')||(c=='.')||((c>='0')&&(c<='9')))
        { 
          Float f,g;
          try 
          { 
            f = new Float(p);
            g = new Float((String)b);
          } 
          catch (NumberFormatException e) 
          { // if either can't be converted, simply compare as strings
            return (J.isString(b) && J.stringEq(p,(String)b)); 
          }
          return (f.floatValue()==g.floatValue());
        }
        else return (J.isString(b) && J.stringEq(p,(String)b));
      }
      // else p is a variable...
      Object r=lookup(p);
      if (r!=null) return unify1(r,b);
      else
      {
        Object obj=subst(b);
        // prevents looping...
        if (J.isString(obj) && J.stringEq(p,(String)obj)) return true;
        if (occurs(p,obj)) return false;
        bind(p,obj); 
        return true;
      }
    }
    else 
    {
      Vector v=(Vector)a; 
      int n=v.size();
      if (!J.isVector(b) || (((Vector)b).size()!=n)) return false;
      Vector w=(Vector)b;
      for (int i=0 ; i<n ; i++)
        if (!unify1(v.elementAt(i),w.elementAt(i))) return false;
      return true;
    }
  }

  boolean occurs(String a,Object b)
  {
    if (J.isString(b)) return J.stringEq(a,(String)b);
    Vector v=(Vector)b;
    int i=0,n=v.size();
    while ((i<n)&&(!occurs(a,v.elementAt(i)))) i++;
    if (i==n) return false;
    else return true;
  }

  public void die(String s)
  {
    System.out.print("Error: ");
    System.out.println(s);
    System.exit(-1);
  }

  public void warn(String s)
  {
    System.out.println("warning: "+s);
  }
}
