/*
 * $Id$
 */
package gui;

import java.awt.Image;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;

/**
 * @author Kai Sterker
 *
 */
public class GrafikCache {
	static private Map grafiken = new HashMap ();
	static private final String PFAD = "grafik/"; 
	
	/**
	 * Lade Grapfik aus Datei
	 * @param datei Dateiname der Grafik
	 * @param name Bezeichnung der Grafik
	 * @return true bei Erfolg, sonst false
	 */
	private static boolean lade (String datei, String name) throws IOException {
		FileInputStream reader = new FileInputStream (datei);
		
		byte[] data = new byte[reader.available()];
		reader.read (data);
		reader.close ();
		
		ImageIcon grafik = new ImageIcon (data);
		grafiken.put (name, grafik);
		
		return true;
	}
	
	/**
	 * Liefere Grafik mit gegebener Bezeichnung aus dem Cache zurueck.
	 * @param name Bezeichnung der Grafik
	 * @return Die Grafik, falls sie existiert, sonst null.
	 */
	public static Image getGrafik (String name) {
		if (grafiken.containsKey (name)) 
			return ((ImageIcon) grafiken.get (name)).getImage ();
		
		try {
			if (lade (PFAD + name + ".gif", name))
				return ((ImageIcon) grafiken.get (name)).getImage ();
		} catch (IOException e) {
			System.out.print ("Kann '" + name + "' nicht finden!");
		}
		return null; 
	}
}
